"""
================
``radiospectra``
================

An open-source Python library for radio spectra in solar physics.

* Homepage: https://sunpy.org
* Documentation: https://docs.radiospectra.org/en/stable/
* Source code: https://github.com/sunpy/radiospectra
"""

from .version import version as __version__

__all__ = ["__version__"]
